<?php

/*
 * Xml cart object to be sent to world pay
*/

class worldpayCart extends CComponent {

    //wp cart details
    private $wpMerchantId;
    private $wpMerchantKey;
    private $wpVersion;
    private $wpCurrency;
    private $wpExponent;
    private $wpServerUrl;
    private $wpXml;
    private $wpInstallationId;
    private $wpSuccessUrl;
    private $wpFailUrl;
    private $certPath;

    //purchase specific
    private $purchaseValue;
    private $purchaseName;
    private $purchaseDesc;
    private $purchaseCount;
    private $purchaseId;
    
    //customer specifc
    private $buyerTitle;
    private $buyerFirst;
    private $buyerLast;
    private $buyerPhone;
    private $buyerEmail;
    private $buyerPostcode;
    private $buyerHouseNo;
    private $buyerAddressTwo;
    private $buyerCity;
    private $buyerCountry;
    private $buyerCardType;

    /* need to rmeove decimal point from value and store it seperately as exponent*/
    /* this function sets both exponent and removes the decimal from purchase value */
    protected function setExponent($value) { 
	
	$dpFromRight = strlen(strrchr($value, '.')) -1;

	switch ($dpFromRight)
        	{
		//ie 5,6,7
		case -1:
  		$this->wpExponent=0;
  		break;
		//5.5, 6.5
		case 1:
  		$this->wpExponent=2;
  		$value .= 0;
  		break;
		//5.55, 6.66
		case 2:
  		$this->wpExponent=2;
  		break;
		}

    $this->purchaseValue = str_replace('.', '', $value);
    }

    /* contructor uses config array to initilaise itself  */
    public function worldpayCart($config) {
	
	foreach ($config as $property => $value) 
		{
	    	$this->$property = $value;
		}
        $this->setExponent($this->purchaseValue);

    }

    public function get_XML() {

        $xml = new xmlBuild('<!DOCTYPE paymentService PUBLIC "-//RBS Worldpay/DTD RBS WorldPay PaymentService v1//EN" "http://dtd.wp3.rbsworldpay.com/paymentService_v1.dtd">');
	
	$xml->Push('paymentService',
			array('version' => $this->wpVersion, 'merchantCode' => $this->wpMerchantId)
			);
	$xml->Push('submit');

	$xml->Push('order',
			array('orderCode' => $this->purchaseId, 'installationId' => $this->wpInstallationId));
   	$xml->Element('description', $this->purchaseDesc);

	$xml->EmptyElement('amount',
			array('value' => $this->purchaseValue, 'currencyCode' => $this->wpCurrency, 'exponent' => $this->wpExponent));

	$xml->Push('orderContent');

/* Use this section to add additional data in CDATA  */	
//	$xml->cdata_open('CDATA', '<div></div>');
	
	$xml->Pop('orderContent');
	$xml->Push('paymentMethodMask');
	$xml->EmptyElement('include',array('code' => "ALL"));
	$xml->EmptyElement('exclude',array('code' => "ALL"));
	$xml->Pop('paymentMethodMask');
    	$xml->Push('shopper');
    	$xml->Element('shopperEmailAddress', $this->buyerEmail);
	$xml->Pop('shopper');
	$xml->Push('shippingAddress');
		$xml->Push('address');
			$xml->Element('firstName', $this->buyerFirst);
			$xml->Element('lastName', $this->buyerLast);
			$xml->Element('street', $this->buyerAddressTwo);
			
			// if number number is purely numerical populate house number
			if (ctype_digit($this->buyerHouseNo) == 1)
					{$xml->Element('houseNumber', $this->buyerHouseNo);}
                                        else {$xml->Element('houseName', $this->buyerHouseNo);}
		
			$xml->Element('postalCode', $this->buyerPostcode);
			$xml->Element('city', $this->buyerCity);
			$xml->Element('countryCode', $this->buyerCountry);
			$xml->Element('telephoneNumber', $this->buyerPhone);
		$xml->Pop('address');
	$xml->Pop('shippingAddress');
	
    $xml->Pop('order'); 
    $xml->Pop('submit');
    $xml->Pop('paymentService');
     
   //  header('Content-type: text/xml');
   //  echo $xml_data->GetXML();
   //  die();
return $xml->GetXML();
}//end getXML

/**
     * Submit a server-to-server request.
     * 
     * @param string $xml_cart the cart's xml
     * @param bool $die whether to die() or not after performing the request,
     *                  defaults to true
     * 
     * @return array with the returned http status code (200 if OK) in index 0 
     *               and the redirect url returned by the server in index 1
     */
function send_worldpay_xml($xml_cart, $die=true) {
      list($status, $body) = $this->SendReq($this->wpServerUrl,
                   $this->GetAuthenticationHeaders(), $xml_cart);
      
      //something went wrong
      if($status != 200) {
          return array($status, $body);
          		} 
      //everything is ok    
          else {
          		//need to append preferred payment method
        		$body .='&preferredPaymentMethod='.urlencode($this->buyerCardType); 
        		//need to append result urls
        		$body .='&successURL=' . urlencode($this->wpSuccessUrl);
        		$body .='&failureURL=' . urlencode($this->wpFailUrl);
        		header('Location: ' . $body);
        if($die) {die($body);}
        else {}
      }
    }//end worldpaycart
    
/**/
function GetAuthenticationHeaders() {
      $headers = array();
      $headers[] = "Content-Type: text/xml; charset=UTF-8";
      $headers[] = "Accept: text/xml; charset=UTF-8";
      return $headers; 
    }//end getAuthenticHeaders

/**/
function SendReq($url, $header_arr, $postargs, $timeout=false) {
      $session = curl_init($url);
       //Yii::log($postargs, 'info'); 
       // Set the POST options.
      curl_setopt($session, CURLOPT_POST, true);
      curl_setopt($session, CURLOPT_USERPWD, $this->wpMerchantId.":".$this->wpMerchantKey);
      curl_setopt($session, CURLOPT_HTTPHEADER, $header_arr);        
      //postargs is the xml
      curl_setopt($session, CURLOPT_POSTFIELDS, $postargs);
      //curl_setopt($session, CURLOPT_HEADER, true);
      curl_setopt($session, CURLOPT_RETURNTRANSFER, true);

      if(!empty($this->certPath) && file_exists($this->certPath)) {
        curl_setopt($session, CURLOPT_SSL_VERIFYPEER, true);
        curl_setopt($session, CURLOPT_CAINFO, $this->certPath);
      }
      else {
        curl_setopt($session, CURLOPT_SSL_VERIFYPEER, false);
      }

      // Do the POST and then close the session
      $response = curl_exec($session);
      
      if (curl_errno($session)) {
 		Yii::log(curl_error($session), 'info');    
    		return array("CURL_ERR", curl_error($session));
      } else
	 {
        curl_close($session);
      }

try{
   		//Yii::log($response, 'info');
		$xml = new SimpleXMLElement($response); 
		
		$value =(string)$xml->reply->orderStatus->reference; 		
		
		//if the server sends an error we'll log and return 400
		if (!isset($value) || $value=="") {
					
					Yii::log($response, 'info');
				        Yii::log($postargs, 'info'); 	
					return array (400, null);
							}				
		//everything is fine
		else 
		return array(200, $value);
		}
	
	catch (Exception $e) {}
	return array (400, null);
}//send requirements
    
} //end class
?>
